/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

/*!
 * \brief class describes camera using imageJ form pc windows
 * \author J. Locatelli
 * \date 11-06-2015
 */

#ifndef DIRECTCAMERA_H
#define DIRECTCAMERA_H

#include "controllers/common/scanlegacy/ParameterizableScan1D.h"
#include "controllers/common/utilities/imageviewer/ImageData.h"
#include "controllers/common/acquisition/ImageGeneratable.h"
#include "drivers/imagingsources/dfk31au03/Dfk31Au03Driver.h"

namespace dif_s {

/*!
 * \class DirectCamera
 * \brief Abstract class describes the scan mesure
 *
 * \par
 *	This class contains mathematics methods. It described sample properties
 */
class DirectCamera: public utilities::ImageData, public acquisition::ImageGeneratable, public controller::Stoppable {

public:

	//! Type of controller
	static const std::string TYPE;

	/*!
	 * \brief Constructor
	 * \param[in] name the name of the experiment controller
	 */
	DirectCamera(const std::string& name);

	/*!
	 * \brief Destructor
	 */
	virtual ~DirectCamera();

	/*!
	 * Properties
	 */
	Property<bool> liveView;
	Property<std::string> cameraPosition;
	Property<std::string> cameraReactor;
	Property<std::string> axisName;
	Property<float64> startedPosition;
	Property<float64> endedPosition;
	Property<float64> stepPosition;
	Property<float64> exposureTime;
	Property<float64, SETPOINT> position;
	Property<string> rgbData;
	Property<int32> xSize;
	Property<int32> ySize;

private:

	/*!
	 * \brief Method called before changing the property value
	 *
	 * This method is called after setting configuration during the creation of controller.
	 */
	virtual void postConfiguration();

	ControllerPtr<scan::ParameterizableScan1D> m_Scan;
	DriverPtr<dfk41au03::Dfk31Au03Driver> m_Driver;

	/*!
	 * \brief Start command
	 *
	 * This command implementation applies chosen voltage and current to the power supply.
	 */
	virtual void start();

	/*!
	 * \brief Stop command
	 *
	 * This command implementation stops.
	 */
	virtual void stop();

	void updateRGBData();
	void updateProgression();

};

}

#endif //DIRECTCAMERA_H
