/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "CAENPksDef.h"
#include "CAENPksState.h"
#include "CAENPksDriver.h"
#include "PerfectCAENPksDriver.h"
#include "RealCAENPksDriver.h"
#include "drivers/global/DriversCommands.h"

using namespace driver;

namespace caen_pks {

const std::string CAENPksDriver::TYPE = "caen_pks";

const std::string CAENPksDriver::WRITE_PARAMETERS_COMMAND = "writeParam";
const std::string CAENPksDriver::RESET_COMMAND = "reset";
const std::string CAENPksDriver::CLEAR_COMMAND = "clear";

/*
 * Constructor
 */
CAENPksDriver::CAENPksDriver(const string& name) {

	driver::Vme::init(name);

	registerStates(new RealCAENPksDriver(this), new PerfectCAENPksDriver(this), new PerfectCAENPksDriver(this));

	// Global properties
	nbChannels.init(this, SAVE, "nb_channels");
    model.init(this, NOSAVE, "model");
    modelName.init(this, SAVE, "model_name");
    firmware.init(this, NOSAVE, "firmware");
	triggerThreshold.init(this, SAVE, "trigger_threshold");
	channelActive.init(this, SAVE, "channel_active");
	channelActiveStr.init(this, NOSAVE, "channel_active_str");

	// Init the device command list
	initCommand(driver::INIT_COMMAND);
	initCommand(driver::READ_INFOS_COMMAND);
	initCommand(WRITE_PARAMETERS_COMMAND);
	initCommand(RESET_COMMAND);
	initCommand(CLEAR_COMMAND);

	// Init functions
	registerFunction(NONE_FUNCTION);

	addressType.update(driver::Vme::A32_ACCESS);
	deviceType = LEAF_DEVICE_TYPE_DEVICE_CONTAINER;

	registerRefresher(nbChannels, &CAENPksDriver::refreshNbChannelsProperty, this);
	registerRefresher(channelActive, &CAENPksDriver::refreshChannelActiveProperty, this);
}

/*
 * Destructor
 */
CAENPksDriver::~CAENPksDriver() {

}

/*
 * execute
 */
void CAENPksDriver::execute(const std::string& aCommand) {

	if ((aCommand != driver::STOP_COMMAND) && (aCommand != driver::STATUS_COMMAND)
			&& (aCommand != driver::READ_COMMAND)) {
		commandProgression = PROGRESSION_UNKNOWNSTATE_DEVICE_CONTAINER;
	}

	CAENPksState* currentState = dynamic_cast<CAENPksState*>(getCurrentState());
	// Check command
	if (aCommand == driver::INIT_COMMAND) {
		// Init command
		currentState->init();
	} else if (aCommand == WRITE_PARAMETERS_COMMAND) {
		// writeParam command
		currentState->writeParam();
		commandProgression = PROGRESSION_END_DEVICE_CONTAINER;
	} else if (aCommand == CLEAR_COMMAND) {
		// clear command
		currentState->clear();
		commandProgression = PROGRESSION_END_DEVICE_CONTAINER;
	} else if (aCommand == RESET_COMMAND) {
		// clear command
		currentState->reset();
		commandProgression = PROGRESSION_END_DEVICE_CONTAINER;
	} else if (aCommand == driver::READ_INFOS_COMMAND) {
		// Info command
		currentState->readInfos();
		commandProgression = PROGRESSION_END_DEVICE_CONTAINER;
	} else {
		//Error bad command
		//Todo , normaly it will be detect it device action driver
	}
}

void CAENPksDriver::refreshNbChannelsProperty(int32 aValue)  throw (CannotSetValue){

	triggerThreshold.resize(aValue);
	channelActive.resize(aValue);
	channelActiveStr.resize(aValue);
}

void CAENPksDriver::refreshChannelActiveProperty(int32 index, int32 aValue) throw (CannotSetValue) {

	switch (aValue) {
	case 0:
		channelActiveStr.set(index, "Inactive");
		break;
	case 1:
		channelActiveStr.set(index, "Active");
		break;
	}
}

}
